#!/usr/bin/python
#import sys
#sys.path.insert(0,'/scratch/mimeau/install-parmes3/')
import parmepy as pp
from parmepy.f2py import fftw2py
import numpy as np
from parmepy.mpi.topology import Cartesian
from parmepy.operator.advection import Advection
from parmepy.operator.stretching import Stretching
from parmepy.operator.poisson import Poisson
from parmepy.operator.diffusion import Diffusion
from parmepy.operator.redistribute import Redistribute
from parmepy.problem.navier_stokes import NSProblem
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from dataTG import dim, nb, pi, NBGHOSTS, ADVECTION_METHOD, VISCOSITY, \
    WITH_PROJ, OUTPUT_FREQ, FILENAME, simu


## ----------- A 3d problem -----------
print " ========= Start Navier-Stokes 3D (Taylor Green benchmark) ========="

## Domain
box = pp.Box(dim, length=[2.0 * pi, 2.0 * pi, 2.0 * pi])

## Global resolution
nbElem = [nb] * dim


## Function to compute TG velocity
def computeVel(x, y, z):
    vx = np.sin(x) * np.cos(y) * np.cos(z)
    vy = - np.cos(x) * np.sin(y) * np.cos(z)
    vz = 0.
    return vx, vy, vz


## Function to compute reference vorticity
def computeVort(x, y, z):
    wx = - np.cos(x) * np.sin(y) * np.sin(z)
    wy = - np.sin(x) * np.cos(y) * np.sin(z)
    wz = 2. * np.sin(x) * np.sin(y) * np.cos(z)
    return wx, wy, wz

## Fields
velo = pp.Field(domain=box, formula=computeVel,
                name='Velocity', isVector=True)
vorti = pp.Field(domain=box, formula=computeVort,
                 name='Vorticity', isVector=True)

## Usual Cartesian topology definition
ghosts = np.ones((box.dimension)) * NBGHOSTS
topo = Cartesian(box, box.dimension, nbElem,
                 ghosts=ghosts)

## Operators
advec = Advection(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method=ADVECTION_METHOD
                  )

stretch = Stretching(velo, vorti,
                     resolutions={velo: nbElem,
                                  vorti: nbElem},
                     topo=topo
                     )

diffusion = Diffusion(vorti,
                      resolution=nbElem,
                      viscosity=VISCOSITY
                     )

poisson = Poisson(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  projection=WITH_PROJ)

## Diagnostics related to the problem

energy = Energy_enstrophy(velo, vorti,
                          topo=topo,
                          viscosity=VISCOSITY,
                          frequency=OUTPUT_FREQ,
                          prefix=FILENAME)

distrAdvStr = Redistribute([vorti, velo], advec, stretch)
distrStrPoiss = Redistribute([vorti], stretch, poisson)
distrPoissStretch = Redistribute([vorti, velo], poisson, stretch)

## Define the problem to solve
pb = NSProblem(operators=[advec, distrAdvStr, stretch, distrStrPoiss,
                          diffusion, poisson, distrPoissStretch],
               simulation=simu, monitors=[energy])

## Setting solver to Problem
pb.setUp()

print 'all topologies', box.topologies


## Solve problem
#poisson.apply(simu)
pb.solve()

## end of time loop ##

# Clean memory buffers
fftw2py.clean_fftw_solver(box.dimension)
