#!/usr/bin/env python
import parmepy
#parmepy.__VERBOSE__ = True
from parmepy.domain.box import Box
from parmepy.gpu import PARMES_REAL_GPU
#parmepy.gpu.CL_PROFILE = True
from parmepy.fields.continuous import Field
from parmepy.operator.advection import Advection
from parmepy.problem.transport import TransportProblem
from parmepy.operator.monitors.printer import Printer
from parmepy.problem.simulation import Simulation
from parmepy.operator.analytic import Analytic
# import math

# def vitesse(x, y):
#     vx = -math.sin(x * math.pi) ** 2 * math.sin(y * math.pi * 2)
#     vy = math.sin(y * math.pi) ** 2 * math.sin(x * math.pi * 2)
#     return vx, vy


# def vitesse(x, y, t=0):
#     vx = -math.sin(x * math.pi) ** 2 * math.sin(y * math.pi * 2) * \
#         math.cos(t * math.pi / 3.)
#     vy = math.sin(y * math.pi) ** 2 * math.sin(x * math.pi * 2) * \
#         math.cos(t * math.pi / 3.)
#     return vx, vy


# def scalaire(x, y):
#     rr = math.sqrt((x - 0.5) ** 2 + (y - 0.75) ** 2)
#     if rr < 0.15:
#         return 1.
#     else:
#         return 0.


dim = 2
boxLength = [1., 1.]
boxMin = [0., 0.]
nbElem = [1025, 1025]

timeStep = 0.025
finalTime = 3.
outputFilePrefix = './res_2D/levelSet_2D_rect_'
outputModulo = 0
simu = Simulation(tinit=0.0, tend=finalTime, timeStep=timeStep)

## Domain
box = Box(dim, length=boxLength, origin=boxMin)

## Fields
scal = Field(domain=box, name='Scalar')
velo = Field(domain=box, name='Velocity', isVector=True)

## Operators
advec = Advection(velo, scal,
                  resolutions={velo: nbElem,
                               scal: nbElem},
                  method='gpu_2k_m4prime',
                  #method='gpu_1k_m6prime',
                  #method='gpu_1k_m8prime',
                  #method='gpu_2k_m4prime',
                  #method='gpu_2k_m6prime',
                  #method='gpu_2k_m8prime',
                  #method='scales'
                  src=['./levelSet2D.cl'],
                  precision=PARMES_REAL_GPU,
                  #precision=PARMES_DOUBLE_GPU,
                  #splittingConfig='o2'
                  #splittingConfig='y_only'
                  #splittingConfig='o2_FullHalf'
                  )
velocity = Analytic(velo,
                    resolutions={velo: nbElem},
                    method='gpu',
                    src=['./levelSet2D.cl'],
                    precision=PARMES_REAL_GPU,
                    )

##Problem
#pb = TransportProblem([advec],simu,
pb = TransportProblem([velocity, advec], simu,
                      monitors=[Printer(fields=[scal],
                                        frequency=outputModulo,
                                        prefix=outputFilePrefix)])

## Setting solver to Problem
pb.setUp()

## Solve problem
pb.solve()

pb.finalize()

th, tt = pb.timer.toString()
for op in pb.operators:
    oth, ott = op.timer.toString()
    th += oth
    tt += ott
print th
print tt

