#!/usr/bin/env python
import parmepy as pp
from parmepy.gpu import PARMES_REAL_GPU, PARMES_DOUBLE_GPU
from parmepy.fields.continuous import Field
from parmepy.operator.advection import Advection
from parmepy.problem.transport import TransportProblem
from parmepy.operator.monitors.printer import Printer
from parmepy.operator.analytic import Analytic
from parmepy.problem.simulation import Simulation
# from math import sin, cos, pi


# def scalaire(x,y,z):
#     return 1.


# def vitesse(x, y, z, t=0):
#     vx = 2. * sin(pi*x)**2*sin(2.*pi*y)*sin(2.*pi*z)*cos(t*pi/3.)
#     vy = -sin(2.*pi*x)*sin(pi*y)**2*sin(2.*pi*z)*cos(t*pi/3.)
#     vz = -sin(2.*pi*x)*sin(2.*pi*y)*sin(pi*z)**2*cos(t*pi/3.)
#     return vx, vy, vz

dim = 3
boxLength = [1., 1., 1.]
boxMin = [0., 0., 0.]
nbElem = 3 * [129]

timeStep = 0.05
finalTime = 3.
outputFilePrefix = './res3D_new/levelSet_3D_'
outputModulo = 1
simu = Simulation(tinit=0.0, tend=finalTime, timeStep=timeStep)

## Domain
box = pp.Box(dim, length=boxLength, origin=boxMin)

## Fields
scal = Field(domain=box, name='Scalar')
velo = Field(domain=box, name='Velocity', isVector=True)

## Operators
advec = Advection(velo, scal,
                  resolutions={velo: nbElem,
                               scal: nbElem},
                  #method='scales',
                  method='gpu_1k_m4prime',
                  #method='gpu_1k_m6prime',
                  #method='gpu_1k_m8prime',
                  #method='gpu_2k_m4prime',
                  #method='gpu_2k_m6prime',
                  #method='gpu_2k_m8prime',
                  src=['./levelSet3D.cl'],
                  precision=PARMES_REAL_GPU,
                  #precision=PARMES_DOUBLE_GPU,
                  splittingConfig='o2'
                  #splittingConfig='o2_FullHalf'
                  )
velocity = Analytic(velo,
                    resolutions={velo: nbElem},
                    method='gpu',
                    src=['./levelSet3D.cl'],
                    precision=PARMES_REAL_GPU,
                    )

##Problem
#pb = TransportProblem([advec],
pb = TransportProblem([velocity, advec], simu,
                      monitors=[Printer(fields=[scal],
                                        frequency=outputModulo,
                                        prefix=outputFilePrefix)])

## Setting solver to Problem
pb.setUp()

## Solve problem
pb.solve()

pb.finalize()

th, tt = pb.timer.toString()
for op in pb.operators:
    oth, ott = op.timer.toString()
    th += oth
    tt += ott
print th
print tt
