"""
Testing parmepy.field.continuous.Field
"""
import parmepy
from parmepy.fields.continuous import Field
from parmepy.domain.box import Box
from parmepy.mpi.topology import Cartesian
import sys


def test_continuous():
    """ Basic continuous field construction """

    print sys.path
    dom = Box()
    print "file ..." , parmepy.__file__
    cf = Field(dom)
    assert cf.isVector is False
    assert cf.domain is dom
    cf = Field(dom, isVector=True)
    assert cf.isVector is True
    assert cf.domain is dom


def test_analytical():
    """Test formula"""
    dom = Box()
    formula = lambda x, y, z: (x * y * z, 1.0, -x * y * z)
    caf = Field(dom, formula)
    assert caf.value(0., 0., 0.) == (0., 1., 0.)
    assert caf.value(1., 1., 1.) == (1., 1., -1.)
    assert caf.value(1., 2., 3.) == (6., 1., -6.)


def test_analytical_reset():
    """Test formula"""
    dom = Box()
    formula = lambda x, y, z: (x * y * z, 1.0, -x * y * z)
    caf = Field(dom, formula)
    assert caf.value(0., 0., 0.) == (0., 1., 0.)
    assert caf.value(1., 1., 1.) == (1., 1., -1.)
    assert caf.value(1., 2., 3.) == (6., 1., -6.)
    form2 = lambda x, y, z, dt: (x * y * z, 1.0, -x * y * z * dt)
    caf.setFormula(form2)
    assert caf.value(0., 0., 0., 0.) == (0., 1., 0.)
    assert caf.value(1., 1., 1., 1.) == (1., 1., -1.)
    assert caf.value(1., 2., 3., 4.) == (6., 1., -24.)


def test_analytical2():
    """Test formula, with topo"""
    dom = Box()
    resol = [9, 9, 9]
    topo = Cartesian(dom, 3, resol)
    formula = lambda x, y, z, t: t
    caf = Field(dom, formula)
    cafd = caf.discretize(topo)
    assert caf.value(0., 0., 0., 1) == formula(0., 0., 0., 1)
    t = 1
    caf.initialize(t)
    assert (cafd[0] == t).all()


def test_analytical3():
    """Test formula, with topo and extra parameters"""
    dom = Box()
    resol = [9, 9, 9]
    topo = Cartesian(dom, 3, resol)
    formula = lambda x, y, z, t, theta: t * theta
    caf = Field(dom, formula)
    theta = 3.2
    caf.setExtraParameters(theta)
    cafd = caf.discretize(topo)
    t = 1.5
    assert caf.value(0., 0., 0., t, theta) == formula(0., 0., 0., t, theta)
    caf.initialize(t)
    assert (cafd[0] == t * theta).all()
    formula = lambda x, y, z, t, theta, R: t * theta * R
    caf2 = Field(dom, formula)
    caf2.setExtraParameters(3.2, 2)
    cafd2 = caf2.discretize(topo)
    caf2.initialize(t)
    assert (cafd2[0] == t * 3.2 * 2).all()
