/**
 * @file builtin_noVec.cl
 * Advection function, basic version
 */

float advection(uint i, float dt, float dx, float invdx, __global float* gvelo);


/**
 * Compute the position of a particle with a RK2 integration scheme. Velocity is linearly interpolated from the global field.
 * Use of builtin OpenCL functions fma and mix.
 *
 * @param i Velocity.
 * @param dt Time step.
 * @param dx Space step.
 * @param invdx 1/dx.
 * @param gvelo Global velocity field.
 * @return Particle position
 *
 * @remark NB_I, NB_II, NB_III : points number in directions from 1st varying index to last.
 */
float advection(uint i, float dt, float dx, float invdx, __global float* gvelo)
{
  float v, 			/* Velocity at point */
    p,				/* Intermediary position */
    vp,				/* Velocity at right point */
    y;				/* Normalized distance to left point */
  int ind, 			/* Integer coordinate (cell id) */
    i_ind,			/* Interpolation left point */
    i_ind_p;			/* Interpolation right point */
  uint line_index = gidY*NB_I+ gidZ*NB_I*NB_II; /* Current 1D problem index */
  v = gvelo[i+line_index];
  p = fma((float)(0.5*dt),v,i*dx);
  ind = convert_int_rtn(p * invdx);
  y = (fma(- convert_float(ind),dx,p))* invdx ;
  i_ind = ((ind + NB_I) % NB_I);
  i_ind_p = ((i_ind + 1) % NB_I);
  return fma(mix(gvelo[i_ind+line_index],
                gvelo[i_ind_p+line_index],y),
            dt,i*dx);
}
