/**
 * @file common.cl
 * Common parameters for advection and remeshing kernels.
 */

inline uint noBC_id(int id);

#ifdef WITH_NOBC
/**
 * Mapping to local memory arrays to avoir banck conflics.
 * 1D buffer is taken as 2D one with wor-items vs. particles.
 *
 * @param id 1D index
 *
 * @return 2D index
 */
inline uint noBC_id(int id){
  return (id%PART_NB_PER_WI)*WI_NB+(id/PART_NB_PER_WI);
}
#else
/**
 * Leave mapping unchanged, 1D.
 *
 * @param id 1D index
 *
 * @return 1D index
 */
inline uint noBC_id(int id){
  return id;
}
#endif

/**
 * Constants for remeshing formulas:
 *   - M4PRIME 1
 *   - L2STAR_C2 2
 *   - L2STAR_C3 3
 *   - L2STAR_C4 4
 *   - L4STAR 5
 *   - L4STAR_C3 6
 *   - L4STAR_C4 7
 *   - L6STAR 8
 *   - L6STAR_C4 9
 *   - L6STAR_C5 10
 *   - L6STAR_C6 11
 *   - M8PRIME 12
 *   - L8STAR 13
 */
#define M4PRIME 1
#define L2STAR_C2 2
#define L2STAR_C3 3
#define L2STAR_C4 4
#define L4STAR 5
#define L4STAR_C3 6
#define L4STAR_C4 7
#define L6STAR 8
#define L6STAR_C4 9
#define L6STAR_C5 10
#define L6STAR_C6 11
#define M8PRIME 12
#define L8STAR 13

/**
 * Shift to left grid point
 */
#if FORMULA == M4PRIME
#define REMESH_SHIFT 1
#elif FORMULA == L2STAR_C2
#define REMESH_SHIFT 1
#elif FORMULA == L2STAR_C3
#define REMESH_SHIFT 1
#elif FORMULA == L2STAR_C4
#define REMESH_SHIFT 1

#elif FORMULA == L4STAR
#define REMESH_SHIFT 2
#elif FORMULA == L4STAR_C3
#define REMESH_SHIFT 2
#elif FORMULA == L4STAR_C4
#define REMESH_SHIFT 2

#elif FORMULA == M8PRIME
#define REMESH_SHIFT 3
#elif FORMULA == L6STAR
#define REMESH_SHIFT 3
#elif FORMULA == L6STAR_C4
#define REMESH_SHIFT 3
#elif FORMULA == L6STAR_C5
#define REMESH_SHIFT 3
#elif FORMULA == L6STAR_C6
#define REMESH_SHIFT 3

#elif FORMULA == L8STAR
#define REMESH_SHIFT 4
#endif

