/**
 * @file advection_noVec.cl
 * Advection kernel, basic version.
 */

/**
 * Computes particles positions from the velocity field.
 * A work-group is handling a 1D problem. Thus, gidY and gidZ are constants among work-items of a work-group.
 * Each work-item computes NB_I/WI_NB particles positions.
 *
 * @param gvelo Velocity.
 * @param ppos Particle position.
 * @param dt Time step.
 * @param min_position Domain lower coordinate.
 * @param dx Space step.
 *
 * @remark NB_I, NB_II, NB_III : points number in directions from 1st varying index to last.
 * @remark WI_NB corresponds to the work-item number.
 */
__kernel void advection_kernel(__global const float* gvelo,
			       __global float* ppos,
			       float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);	/* OpenCL work-itme global index (X) */
  uint gidY = get_global_id(1); /* OpenCL work-itme global index (Y) */
  uint gidZ = get_global_id(2); /* OpenCL work-itme global index (Z) */
  float invdx = 1.0/dx;		/* Space step inverse */
  uint i;			/* Particle index in 1D problem */

  for(i=gidX; i<NB_I; i+=WI_NB)
    {
      ppos[i+gidY*NB_I+gidZ*NB_I*NB_II] = advection(i, dt, dx, invdx, gvelo);
    }
}
