"""
@file parmepy.gpu.tests.test_copy
Testing copy kernels.
"""
from parmepy.gpu import PARMES_REAL_GPU, cl
from parmepy.constants import ORDER, np
from parmepy.gpu.tools import get_opencl_environment
from parmepy.gpu.gpu_kernel import KernelLauncher


def test_copy2D():
    resolution = (256, 256)
    cl_env = get_opencl_environment(0, 0, 'gpu', PARMES_REAL_GPU, resolution)
    vec = 2
    src_copy = 'kernels/copy.cl'
    build_options = ""
    build_options += cl_env.default_sizes_constants[0]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution[0] / 2),
           int(resolution[1] / 2))
    lwi = (8, 8)
    prg = cl_env.build_src(src_copy, build_options, vec)
    copy = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    data_in = np.asarray(np.random.random(resolution),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    data_gpu_in = cl.Buffer(cl_env.ctx,
                            cl.mem_flags.READ_WRITE,
                            size=data_in.nbytes)
    data_gpu_out = cl.Buffer(cl_env.ctx,
                             cl.mem_flags.READ_WRITE,
                             size=data_out.nbytes)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl_env.queue.finish()

    copy(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_gpu_in.release()
    data_gpu_out.release()


def test_copy2D_rect():
    resolution = (256, 512)
    resolutionT = (512, 256)
    cl_env = get_opencl_environment(0, 0, 'gpu', PARMES_REAL_GPU, resolution)
    vec = 2
    src_copy = 'kernels/copy.cl'
    build_options = ""
    build_options += cl_env.default_sizes_constants[0]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution[0] / 2),
           int(resolution[1] / 2))
    lwi = (8, 8)
    prg = cl_env.build_src(src_copy, build_options, vec)
    copy_x = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    build_options = ""
    build_options += cl_env.default_sizes_constants[1]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution[1] / 2),
           int(resolution[0] / 2))
    lwi = (8, 8)
    prg = cl_env.build_src(src_copy, build_options, vec)
    copy_y = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    data_in = np.asarray(np.random.random(resolution),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    data_gpu_in = cl.Buffer(cl_env.ctx,
                            cl.mem_flags.READ_WRITE,
                            size=data_in.nbytes)
    data_gpu_out = cl.Buffer(cl_env.ctx,
                             cl.mem_flags.READ_WRITE,
                             size=data_out.nbytes)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_out, data_out)
    cl_env.queue.finish()

    copy_x(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_in = np.asarray(np.random.random(resolutionT),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_out, data_out)
    cl_env.queue.finish()

    copy_y(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_gpu_in.release()
    data_gpu_out.release()


def test_copy3D():
    resolution = (64, 64, 64)
    cl_env = get_opencl_environment(0, 0, 'gpu', PARMES_REAL_GPU, resolution)
    vec = 4
    src_copy = 'kernels/copy.cl'
    build_options = ""
    build_options += cl_env.default_sizes_constants[0]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution[0] / 4),
           int(resolution[1] / 2),
           int(resolution[2]))
    lwi = (4, 8, 1)

    # Build code
    prg = cl_env.build_src(src_copy, build_options, vec)
    init_copy = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    data_in = np.asarray(np.random.random(resolution),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    data_gpu_in = cl.Buffer(cl_env.ctx,
                            cl.mem_flags.READ_WRITE,
                            size=data_in.nbytes)
    data_gpu_out = cl.Buffer(cl_env.ctx,
                             cl.mem_flags.READ_WRITE,
                             size=data_out.nbytes)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)

    cl_env.queue.finish()
    init_copy(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_gpu_in.release()
    data_gpu_out.release()


def test_copy3D_rect():
    resolution_x = (16, 32, 64)
    resolution_y = (32, 16, 64)
    resolution_z = (64, 16, 32)
    cl_env = get_opencl_environment(0, 0, 'gpu', PARMES_REAL_GPU, resolution_x)
    vec = 4
    src_copy = 'kernels/copy.cl'

    build_options = ""
    build_options += cl_env.default_sizes_constants[0]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution_x[0] / 4),
           int(resolution_x[1] / 2),
           int(resolution_x[2]))
    lwi = (4, 8, 1)
    prg = cl_env.build_src(src_copy, build_options, vec)
    init_copy_x = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    build_options = ""
    build_options += cl_env.default_sizes_constants[1]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution_x[1] / 4),
           int(resolution_x[0] / 2),
           int(resolution_x[2]))
    lwi = (4, 8, 1)
    prg = cl_env.build_src(src_copy, build_options, vec)
    init_copy_y = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    build_options = ""
    build_options += cl_env.default_sizes_constants[2]
    build_options += " -D TILE_DIM_COPY=16"
    build_options += " -D BLOCK_ROWS_COPY=8"
    gwi = (int(resolution_x[2] / 4),
           int(resolution_x[0] / 2),
           int(resolution_x[1]))
    lwi = (4, 8, 1)
    prg = cl_env.build_src(src_copy, build_options, vec)
    init_copy_z = KernelLauncher(prg.copy, cl_env.queue, gwi, lwi)

    data_in = np.asarray(np.random.random(resolution_x),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    data_gpu_in = cl.Buffer(cl_env.ctx,
                            cl.mem_flags.READ_WRITE,
                            size=data_in.nbytes)
    data_gpu_out = cl.Buffer(cl_env.ctx,
                             cl.mem_flags.READ_WRITE,
                             size=data_out.nbytes)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_out, data_out)

    cl_env.queue.finish()
    init_copy_x(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_in = np.asarray(np.random.random(resolution_y),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_out, data_out)
    cl_env.queue.finish()
    init_copy_y(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_in = np.asarray(np.random.random(resolution_z),
                         dtype=PARMES_REAL_GPU, order=ORDER)
    data_out = np.empty_like(data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_in, data_in)
    cl.enqueue_copy(cl_env.queue, data_gpu_out, data_out)
    cl_env.queue.finish()
    init_copy_z(data_gpu_in, data_gpu_out)
    cl_env.queue.finish()
    cl.enqueue_copy(cl_env.queue, data_out, data_gpu_out)
    cl_env.queue.finish()
    assert np.allclose(data_out, data_in)

    data_gpu_in.release()
    data_gpu_out.release()
