from parmepy.gpu.tools import get_opencl_environment, \
    get_opengl_shared_environment
from parmepy.gpu import PARMES_REAL_GPU, PARMES_DOUBLE_GPU


def test_queue_unique_creation():
    """
    Testing that only one queue is created when multiples calls to get
    an environment.
    """
    cl_env = get_opencl_environment(0, 0, 'gpu',
                                    PARMES_REAL_GPU, None)
    cl_env_id = id(cl_env)
    cl_envb = get_opencl_environment(0, 0, 'gpu',
                                     PARMES_REAL_GPU, None)
    cl_envb_id = id(cl_envb)
    assert cl_env_id == cl_envb_id


def test_parse_src_expand_floatN():
    """
    """
    import StringIO
    cl_env = get_opencl_environment(0, 0, 'gpu',
                                    PARMES_REAL_GPU, None)
    str_as_src = """
   vstore__N__((float__N__)(gscal_loc[noBC_id(i+__NN__,nb_part)],
   ), (i + gidY*WIDTH)/__N__, gscal);
   """
    parsed_str_as_src = """
   vstore4((float4)(gscal_loc[noBC_id(i+0,nb_part)],gscal_loc[noBC_id(i+1,nb_part)],gscal_loc[noBC_id(i+2,nb_part)],gscal_loc[noBC_id(i+3,nb_part)]
   ), (i + gidY*WIDTH)/4, gscal);
   """
    buf = StringIO.StringIO(str_as_src)
    res = cl_env.parse_file(buf, n=4)
    assert len(parsed_str_as_src) == len(res)
    for s1, s2 in zip(parsed_str_as_src, res):
        assert s1 == s2


def test_parse_src_expand():
    """
    """
    import StringIO
    cl_env = get_opencl_environment(0, 0, 'gpu',
                                    PARMES_REAL_GPU, None)
    str_as_src = """
    gvelo_loc[noBC_id(i+__NN__,nb_part)] = v.s__NN__;
    """
    parsed_str_as_src = """
    gvelo_loc[noBC_id(i+0,nb_part)] = v.s0;
    gvelo_loc[noBC_id(i+1,nb_part)] = v.s1;
    gvelo_loc[noBC_id(i+2,nb_part)] = v.s2;
    gvelo_loc[noBC_id(i+3,nb_part)] = v.s3;
    """
    buf = StringIO.StringIO(str_as_src)
    res = cl_env.parse_file(buf, n=4)
    assert len(parsed_str_as_src) == len(res)
    for s1, s2 in zip(parsed_str_as_src, res):
        assert s1 == s2

