# -*- coding: utf-8 -*-
"""
@file euler.py

Forward Euler method.
"""
from parmepy.numerics.integrators.odesolver import ODESolver


class Euler(ODESolver):
    """
    ODESolver implementation for solving an equation system
    with forward Euler method.
    y'(t) = f(t,y)

    y(t_{n+1}) = y(t_n) + dt*f(y(t_n))

    """
    def __init__(self, dim, f=None):
        """
        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        """
        ODESolver.__init__(self, dim, f)
        self.name = 'euler'

    def __call__(self, y, dt, t):

        result = self.f(t, y)
        return [y[i] + dt * result[i] for i in xrange(self._dim)]

    def __str__(self):
        """ToString method"""
        return "Euler Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : Euler"
    print Euler.__doc__
