"""
@file odesolver.py

Classes to describe (time) integrators.
"""
from abc import ABCMeta, abstractmethod
from parmepy.constants import np
from parmepy.numerics.method import NumMethod


class ODESolver(NumMethod):
    """
    Abstract description for ODE solvers.
    Solve the system :
    \f[ \frac{dy(t)}{dt} = f(t,y) \f]

    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, dim, f=None):
        """
        @param dim, vector field (f) dimension (number of component)
        @param f : function f(t,y), right hand side of the equation to solve.
        WARNING : y arg must be a list of numpy arrays and f must return
        a list of numpy arrays.
        """
        super(ODESolver, self).__init__()
        ## RHS.
        if f is not None:
            self.f = f
        else:
            self.f = lambda t, y: [np.zeros_like(y[i]) for i in xrange(dim)]

        self._dim = dim

    @abstractmethod
    def __call__(self, y, dt, t):
        """
        @param y : position at time t.
        @param t : current time.
        @param dt : time step.
        @return y at t+dt.
        """

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ODESolver (abstract)"
    print ODESolver.__doc__

