"""
@file runge_kutta3.py

RK3 method interface.
"""
from parmepy.numerics.integrators.odesolver import ODESolver


class RK3(ODESolver):
    """
    ODESolver implementation for solving an equation system with RK3 method.
    """
    def __init__(self, dim, f=None):
        """
        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        """
        ODESolver.__init__(self, dim, f)
        self.name = 'RK3'

    def __call__(self, y, dt, t):

        # k1 = f(t,y)
        # k2 = f(t+dt/3, y + dt/3 *k1))
        # k3 = f(t + 2/3 *dt , y + 2/3 dt * k2))
        # result = y + 0.25 * dt * (k1 + 3 * k3)

        assert len(y) == self._dim

        k1 = self.f(t, y)

        # k2 ... in k3
        k3 = [y[i] + dt / 3 * k1[i] for i in xrange(self._dim)]
        k3 = self.f(t + dt / 3., k3)
        # k3 ...
        k3 = [y[i] + 2 * dt / 3 * k3[i] for i in xrange(self._dim)]
        k3 = self.f(t + dt * 2. / 3., k3)
        # result = y(t,x,y,z)] + 1/4 [ k1 + 3 k3]
        return [y[i] + 0.25 * dt * (k1[i] + 3 * k3[i])
                for i in xrange(self._dim)]

    def __str__(self):
        """ToString method"""
        return "RK3 Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : RK3"
    print RK3.__doc__
