# -*- coding: utf-8 -*-
import time
import parmepy as pp
from parmepy.mpi.topology import Cartesian
from parmepy.numerics.integrators.euler import Euler
from parmepy.numerics.integrators.runge_kutta2 import RK2
from parmepy.numerics.integrators.runge_kutta3 import RK3
from parmepy.numerics.integrators.runge_kutta4 import RK4
from math import *
import numpy as np
import copy
import matplotlib.pyplot as plt
import numpy.testing as npt
from parmepy.constants import PARMES_REAL, ORDER


def analyticalSolution(t, x, y, z):
    sx = (t * np.exp(t) + 1.) * np.exp(-t)
    sy = (t * np.exp(t) + 1.) * np.exp(-t)
    sz = (t * np.exp(t) + 1.) * np.exp(-t)
    return sx, sy, sz

def f (t, u):
    fx = - u[0][:,:,:] + t + 1.
    fy = - u[1][:,:,:] + t + 1.
    fz = - u[2][:,:,:] + t + 1.
    return fx , fy ,fz

def testIntegratorEDO():
    ## Parameters
    nb = 33
    dim = 3
    boxLength = [4. * np.pi, 4. * np.pi, 4. * np.pi]
    boxMin = [- 2. * np.pi,- 2. * np.pi,- 2. * np.pi]

    timeStep = 0.1
    finalTime = 10 * timeStep
    multi = 2.
    maxerror=0.

    t0 = time.time()
    t = 0.

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Solver creation
    topo3D = Cartesian(domain=box, dim=3, 
                       globalMeshResolution=[nb, nb, nb])
    topo3D.setUp()

    Result = np.asarray([np.zeros([nb-1, nb-1, nb-1], 
                        dtype=PARMES_REAL, order=ORDER) 
                        for d in xrange(3)])

    Analvorty = np.asarray([np.zeros([nb-1, nb-1, nb-1], 
                           dtype=PARMES_REAL, order=ORDER) 
                           for d in xrange(3)])

    taille = int(finalTime / timeStep) + 1
    compteur = 0
    dt = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
    errEuler = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
    errRK2 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
    errRK3 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
    errRK4 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)

    UEuler = np.asarray([np.zeros([nb-1, nb-1, nb-1], dtype=PARMES_REAL, 
                         order=ORDER) for d in xrange(dim)])
    URK2 = np.asarray([np.zeros([nb-1, nb-1, nb-1], dtype=PARMES_REAL, 
                       order=ORDER) for d in xrange(dim)])
    URK3 = np.asarray([np.zeros([nb-1, nb-1, nb-1], dtype=PARMES_REAL, 
                       order=ORDER) for d in xrange(dim)])
    URK4 = np.asarray([np.zeros([nb-1, nb-1, nb-1], dtype=PARMES_REAL, 
                       order=ORDER) for d in xrange(dim)])

    UEuler[:,:,:,:] = 1.
    URK2[:,:,:,:] = 1.
    URK3[:,:,:,:] = 1.
    URK4[:,:,:,:] = 1.

#    Analvorty[:,:,:,:] = np.vectorize(analyticalSolution)(timeStep,
#                                      topo3D.mesh.coords[0], \
#                                      topo3D.mesh.coords[1], \
#                                      topo3D.mesh.coords[2])[:]
    v_formula = np.vectorize(analyticalSolution)
    Analvorty[0], Analvorty[1], Analvorty[2] = \
                    v_formula(timeStep,topo3D.mesh.coords[0],
                              topo3D.mesh.coords[1],
                              topo3D.mesh.coords[2])

    while t < finalTime :
        print "t", t

    # Euler
        method = Euler
        methodInt = method(f)
        Result = methodInt(f, t , timeStep , UEuler)
        UEuler = Result
        errEuler[compteur] = np.max(abs(Analvorty[:, :, :, :] 
                                    - Result[:, :, :, :]))

    #  RK2
        method= RK2
        methodInt = method(f)
        Result = methodInt(f, t , timeStep , URK2)
        URK2 = Result
        errRK2[compteur] = np.max(abs(Analvorty[:, :, :, :] 
                                  - Result[:, :, :, :]))

    #  RK3
        method= RK3
        methodInt = method(f)
        Result = methodInt(f, t , timeStep , URK3)
        URK3 = Result
        errRK3[compteur] = np.max(abs(Analvorty[:, :, :, :] 
                                  - Result[:, :, :, :]))

    #  RK4
        method= RK4
        methodInt = method(f)
        Result = methodInt(f, t , timeStep , URK4)
        URK4 = Result
        errRK4[compteur] = np.max(abs(Analvorty[:, :, :, :] 
                                  - Result[:, :, :, :]))

        dt[compteur] = t

        t = t + timeStep

        Analvorty[0], Analvorty[1], Analvorty[2] = \
                    v_formula(t + timeStep,topo3D.mesh.coords[0],
                              topo3D.mesh.coords[1],
                              topo3D.mesh.coords[2])
        compteur = compteur + 1

    # Check the convergence order of each time integration scheme :
    npt.assert_array_less(errEuler, timeStep)
    npt.assert_array_less(errRK2, timeStep ** 2)
    npt.assert_array_less(errRK3, timeStep ** 3)
    npt.assert_array_less(errRK4, timeStep ** 4)

    ## Table of error values
#    print "erreur Euler", errEuler
#    print "erreur RK2", errRK2
#    print "erreur RK3", errRK3
#    print "erreur RK4", errRK4

    ## Plot error_scheme vs time
#    plt.figure(1)
#    plt.subplot(211)
#    plt.xlabel('dt')
#    plt.xscale('log')
#    plt.ylabel('Erreur')
#    plt.yscale('log')
#    plt.plot(dt, errEuler, '+-',dt ,errRK2, '+-' ,dt, errRK3, '+-',dt,errRK4, '+-')
#    plt.legend([u"Euler", u"RK2", u"RK3", u"RK4"], loc=4)
#    plt.subplot(212)
#    plt.xlabel('dt')
#    plt.ylabel('Erreur')
#    plt.plot(dt, errEuler, '+-',dt ,errRK2, '+-' ,dt, errRK3, '+-',dt,errRK4, '+-')
#    plt.legend([u"Euler", u"RK2", u"RK3", u"RK4"], loc=4)

#    plt.show()

    t1 = time.time()

    print "\n"
#    print "Total time : ", tf - t0, "sec (CPU)"
#    print "Init time : ", t1 - t0, "sec (CPU)"
#    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    testIntegratorEDO()
