# -*- coding: utf-8 -*-
"""
@file scales_advection.py
Discrete Advection operator based on scales library (Jean-Baptiste)

"""
from parmepy.f2py import scales2py
from discrete import DiscreteOperator
from parmepy.constants import debug, np
from parmepy.tools.timers import timed_function


class ScalesAdvection(DiscreteOperator):
    """
    Advection process, based on scales library.
    """

    @debug
    def __init__(self, velocity, advectedFields, method=''):
        """
        Constructor.
        @param velocity discrete field
        @param advectedFields : list of discrete fields to be advected
        @param method : Method used.
        """
        ## Advection velocity
        self.velocity = velocity
        variables = [self.velocity]
        if not isinstance(advectedFields, list):
            self.advectedFields = [advectedFields]
        else:
            self.advectedFields = advectedFields
        # update the list of discrete variables for this operator
        [variables.append(advecF) for advecF in self.advectedFields]

        DiscreteOperator.__init__(self, variables, method,
                                  name="SCALES Advection")

        self.input = [self.velocity]
        self.output = list(variables).remove(self.velocity)

    @debug
    @timed_function
    def apply(self, simulation=None):
        if simulation is None:
            raise ValueError("Missing simulation value for computation.")

        dt = simulation.timeStep

        for adF in self.advectedFields:
            for d in xrange(adF.nbComponents):
                adF[d][...] = scales2py.solve_advection(
                    dt, self.velocity.data[0],
                    self.velocity.data[1],
                    self.velocity.data[2],
                    adF[d][...])

    def finalize(self):
        """
        \todo check memory deallocation in scales???
        """
        DiscreteOperator.finalize(self)

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ScalesAdvection"
    print ScalesAdvection.__doc__
