# -*- coding: utf-8 -*-
import parmepy as pp
import numpy as np


def testPenalScal3D():
    from parmepy.numerics.differential_operations import Curl

    nb = 33
    Lx = Ly = Lz = 2.*np.pi
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], 
                 origin=[0., 0., 0.])
    resol = [nb, nb, nb]
    velo = Field(domain=dom, formula=self.vitesse, 
                 name='Velocity', isVector=True)
    topo = velo.discreteFields.keys()[0]
    curl = Curl(velo.topology, 'FD_order4')

    def testOperatorCurl(self):
        # Continuous fields and operator declaration
        self.velo = AnalyticalField(domain=self.box, formula=self.vitesse, name='Velocity', vector=True)
        self.curl = DifferentialOperator(self.velo, self.velo, choice='curl')
        # Topology definition
        self.topo3D = CartesianTopology(domain=self.box, resolution=self.nbPts, dim=self.dim, ghosts=[2,2,2])
        self.topo3DnoG = CartesianTopology(domain=self.box, resolution=self.nbPts, dim=self.dim, ghosts=[0,0,0])
        self.result = [np.ones((self.topo3D.mesh.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(self.dim)]
        # Fields and operator discretization
        self.velo.discretize(self.topo3D)
        self.velo.initialize()
        self.curl.discretize(self.velo.discreteField[self.velo._fieldId], self.velo.discreteField[self.velo._fieldId], topology=self.topo3D)
        self.result = self.curl.discreteOperator.apply()
        self.FinalTime = 0.02
        self.anal = np.vectorize(self.vorticite)(self.topo3DnoG.mesh.coords[0], \
                                                     self.topo3DnoG.mesh.coords[1], \
                                                     self.topo3DnoG.mesh.coords[2])
        # Comparison with analytical solution 
#        print "max:",np.max(abs(self.anal[0]-self.result[0]))
        ind0a = self.topo3D.ghosts[0]
        ind0b = self.topo3D.mesh.resolution[0]-self.topo3D.ghosts[0]
        ind1a = self.topo3D.ghosts[1]
        ind1b = self.topo3D.mesh.resolution[1]-self.topo3D.ghosts[1]
        ind2a = self.topo3D.ghosts[2]
        ind2b = self.topo3D.mesh.resolution[2]-self.topo3D.ghosts[2]
        npt.assert_array_less(abs(self.anal[0] - \
                                            self.result[0][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]), self.e)
#        print "max:",np.max(abs(self.anal[1]-self.result[1]))
        npt.assert_array_less(abs(self.anal[1] - \
                                            self.result[1][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]), self.e)
#        print "max:",np.max(abs(self.anal[2]-self.result[2]))
        npt.assert_array_less(abs(self.anal[2] - \
                                            self.result[2][ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]), self.e)

def vitesse(self, x, y, z):
#    amodul = np.cos(np.pi*self.t/3)
#    pix = np.pi*x
#    piy = np.pi*y
#    piz = np.pi*z
#    pi2x = 2.*pix
#    pi2y = 2.*piy
#    pi2z = 2.*piz
#    vx = 2.*np.sin(pix)*np.sin(pix)*np.sin(pi2y)*np.sin(pi2z)*amodul
#    vy = -np.sin(pi2x)*np.sin(piy)*np.sin(piy)*np.sin(pi2z)*amodul
#    vz = -np.sin(pi2x)*np.sin(piz)*np.sin(piz)*np.sin(pi2y)*amodul
#    vx = np.cos(y)
#    vy = np.cos(z)
#    vz = np.cos(x)
    vx = np.cos(y) * np.cos(z)
    vy = np.cos(z) * np.cos(x)
    vz = np.cos(x) * np.cos(y)
    return vx, vy, vz

def vorticite(self, x, y, z):
#    amodul = np.cos(np.pi*self.t/3)
#    pix = np.pi*x
#    piy = np.pi*y
#    piz = np.pi*z
#    pi2x = 2.*pix
#    pi2y = 2.*piy
#    pi2z = 2.*piz
#    wx = 2.* np.pi * np.sin(pi2x) * amodul*( - np.cos(pi2y)*np.sin(piz)*np.sin(piz)+ np.sin(piy)*np.sin(piy)*np.cos(pi2z) )
#    wy = 2.* np.pi * np.sin(pi2y) * amodul*( 2.*np.cos(pi2z)*np.sin(pix)*np.sin(pix)+ np.sin(piz)*np.sin(piz)*np.cos(pi2x) )
#    wz = -2.* np.pi * np.sin(pi2z) * amodul*( np.cos(pi2x)*np.sin(piy)*np.sin(piy)+ np.sin(pix)*np.sin(pix)*np.cos(pi2y) )
#    wx = np.sin(z)
#    wy = np.sin(x)
#    wz = np.sin(y)
    wx = np.cos(x) * (np.sin(z) - np.sin(y))
    wy = np.cos(y) * (np.sin(x) - np.sin(z))
    wz = np.cos(z) * (np.sin(y) - np.sin(x))
    return wx, wy, wz
