# -*- coding: utf-8 -*-
import parmepy as pp
import numpy as np
from parmepy.fields.continuous import Field
from parmepy.operator.stretching import Stretching
from parmepy.problem.simulation import Simulation


def computeVel(x, y, z):
    amodul = np.cos(np.pi * 1. / 3.)
    pix = np.pi * x
    piy = np.pi * y
    piz = np.pi * z
    pi2x = 2. * pix
    pi2y = 2. * piy
    pi2z = 2. * piz
    vx = 2. * np.sin(pix) * np.sin(pix) \
        * np.sin(pi2y) * np.sin(pi2z) * amodul
    vy = - np.sin(pi2x) * np.sin(piy) \
        * np.sin(piy) * np.sin(pi2z) * amodul
    vz = - np.sin(pi2x) * np.sin(piz) \
        * np.sin(piz) * np.sin(pi2y) * amodul
    return vx, vy, vz


def computeVort(x, y, z):
    amodul = np.cos(np.pi * 1. / 3.)
    pix = np.pi * x
    piy = np.pi * y
    piz = np.pi * z
    pi2x = 2. * pix
    pi2y = 2. * piy
    pi2z = 2. * piz
    wx = 2. * np.pi * np.sin(pi2x) * amodul *\
        (- np.cos(pi2y) * np.sin(piz) * np.sin(piz)
         + np.sin(piy) * np.sin(piy) * np.cos(pi2z))

    wy = 2. * np.pi * np.sin(pi2y) * amodul *\
        (2. * np.cos(pi2z) * np.sin(pix) * np.sin(pix)
         + np.sin(piz) * np.sin(piz) * np.cos(pi2x))

    wz = -2. * np.pi * np.sin(pi2z) * amodul *\
        (np.cos(pi2x) * np.sin(piy) * np.sin(piy)
         + np.sin(pix) * np.sin(pix) * np.cos(pi2y))

    return wx, wy, wz


def test_Stretching():
    # Parameters
    nb = 33
    dim = 3
    boxLength = [1., 1., 1.]
    boxMin = [0., 0., 0.]
    nbElem = [nb, nb, nb]

    timeStep = 0.05

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Fields
    velo = Field(
        domain=box, formula=computeVel,
        name='Velocity', isVector=True)
    vorti = Field(
        domain=box, formula=computeVort,
        name='Vorticity', isVector=True)

    velo.initialize()
    vorti.initialize()
    ## Operators
    stretch = Stretching(velo, vorti,
                         resolutions={velo: nbElem,
                                      vorti: nbElem},
                         method='FD_C4_RK3')
    stretch.setUp()
    simulation = Simulation(0, timeStep, 20)
    stretch.apply(simulation)
