"""
@file parmepy.operator.tests.test_analytic
Testing analytic operator.
"""
import numpy as np
from parmepy.domain.box import Box
from parmepy.fields.continuous import Field
from parmepy.operator.analytic import Analytic
from parmepy.problem.simulation import Simulation


def test_scalar_formula():
    f = lambda x, y, z, t: x - 0.1 * y + 10. * z * z * t
    box = Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])
    field = Field(domain=box, name='Scalar')
    a = Analytic(field, formula=f,
                 resolutions={field: [33, 33, 33], },
                 )
    a.setUp()
    df_id = id(field.discreteFields.values()[0].data[0])
    a.apply(Simulation(10., 0.1, 1))
    # Testing data pointer unchanged
    assert df_id == id(field.discreteFields.values()[0].data[0])
    topo = field.discreteFields.keys()[0]
    coords = topo.mesh.coords
    res = np.vectorize(f)(*(coords + (10.,)))
    assert np.allclose(res, field.discreteFields.values()[0].data[0])


def test_vector_formula():
    f = lambda x, y, z, t: (x, 0.1 * y, 10. * z * z)
    box = Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])
    field = Field(domain=box, name='Vector', isVector=True)
    a = Analytic(field, formula=f,
                 resolutions={field: [33, 33, 33], },
                 )
    a.setUp()
    df_id = id(field.discreteFields.values()[0].data[0])
    df_id1 = id(field.discreteFields.values()[0].data[1])
    df_id2 = id(field.discreteFields.values()[0].data[2])
    a.apply(Simulation(10., 0.1, 1))
    # Testing data pointer unchanged
    assert df_id == id(field.discreteFields.values()[0].data[0])
    assert df_id1 == id(field.discreteFields.values()[0].data[1])
    assert df_id2 == id(field.discreteFields.values()[0].data[2])
    topo = field.discreteFields.keys()[0]
    coords = topo.mesh.coords
    res0, res1, res2 = np.vectorize(f)(*(coords + (10.,)))
    assert np.allclose(res0, field.discreteFields.values()[0].data[0])
    assert np.allclose(res1, field.discreteFields.values()[0].data[1])
    assert np.allclose(res2, field.discreteFields.values()[0].data[2])

