# -*- coding: utf-8 -*-
import parmepy as pp
import numpy as np
from parmepy.domain.obstacle.cylinder2d import SemiCylinder2D
from parmepy.operator.penalization import Penalization
from parmepy.fields.continuous import Field
from parmepy.problem.simulation import Simulation
from parmepy.domain.obstacle.plates import Plates


def computeVel(x, y, ):
    return 1, 1


def computeScal(x, y, t):
    return 1


def testPenalScal2D():

    nb = 33
    Lx = Ly = 2
    dom = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
    resol = [nb, nb]
    scal = Field(domain=dom, name='Scalar')
    hcyl = SemiCylinder2D(dom, position=[0., 0.], radius=0.5,
                          porousLayers=[0.13])
    plates = Plates(dom, normal_dir=1, epsilon=0.1)
    penal = Penalization(scal, [hcyl, plates], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation())
    scalRef = Field(domain=dom, name='ScalarRef')
    scalRef.discretize(topo)
    scalRef.load('ref_scal2D_PenalHspherePlane', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalScal3D():
    from parmepy.domain.obstacle.sphere import HemiSphere

    nb = 33
    Lx = Ly = Lz = 2
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[-1., -1., -1.])
    resol = [nb, nb, nb]
    scal = Field(domain=dom, name='Scalar')
    hsphere = HemiSphere(dom, position=[0., 0., 0.],
                         radius=0.5, porousLayers=[0.13])

    plates = Plates(dom, normal_dir=0, epsilon=0.1)
    penal = Penalization(scal, [hsphere, plates], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation())
    scalRef = Field(domain=dom, name='ScalarRef')
    scd = scalRef.discretize(topo)
    scalRef.load('ref_scal3D_PenalHspherePlane', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalVec2D():

    nb = 33
    Lx = Ly = 2
    dom = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
    resol = [nb, nb]
    velo = Field(domain=dom, name='Velo', isVector=True)
    hcyl = SemiCylinder2D(dom, position=[0., 0.], radius=0.5,
                          porousLayers=[0.13])
    plates = Plates(dom, normal_dir=1, epsilon=0.1)
    penal = Penalization(velo, [hcyl, plates], coeff=[1e6, 10],
                         resolutions={velo: resol})
    penal.setUp()
    topo = velo.discreteFields.keys()[0]
    velod = velo.discreteFields[topo]
    velod[0] = 128
    velod[1] = 12
    penal.apply(Simulation())
    veloRef = Field(domain=dom, name='VeloRef', isVector=True)
    veloRef.discretize(topo)
    veloRef.load('ref_velo2D_PenalHspherePlane', fieldname='VeloRef')
    assert (veloRef.norm() == velo.norm()).all()


def testPenalVec3D():
    from parmepy.domain.obstacle.sphere import HemiSphere

    nb = 33
    Lx = Ly = Lz = 2
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[-1., -1., -1.])
    resol = [nb, nb, nb]
    velo = Field(domain=dom, name='Velo', isVector=True)
    hsphere = HemiSphere(dom, position=[0., 0., 0.],
                         radius=0.5, porousLayers=[0.13])

    plates = Plates(dom, normal_dir=0, epsilon=0.1)
    penal = Penalization(velo, [hsphere, plates], coeff=[1e6, 10],
                         resolutions={velo: resol})
    penal.setUp()
    topo = velo.discreteFields.keys()[0]
    velod = velo.discreteFields[topo]
    velod[0] = 128
    velod[1] = 12
    velod[2] = 4.3
    penal.apply(Simulation())
    veloRef = Field(domain=dom, name='VeloRef', isVector=True)
    veloRef.discretize(topo)
    veloRef.load('ref_velo3D_PenalHspherePlane', fieldname='VeloRef')
    assert (veloRef.norm() == velo.norm()).all()

