# -*- coding: utf-8 -*-

import parmepy as pp
from parmepy.operator.poisson import Poisson
from parmepy.operator.analytic import Analytic
from parmepy.problem.simulation import Simulation
import numpy as np
import math
pi = math.pi
sin = np.sin
cos = np.cos

## Physical Domain description
dim = 3
LL = 2 * pi * np.ones((dim))
# formula to compute initial vorticity field
coeff = 4 * pi ** 2 * (LL[1] ** 2 * LL[2] ** 2 + LL[0] ** 2 * LL[2] ** 2 +
                       LL[0] ** 2 * LL[1] ** 2) / (LL[0] ** 2 * LL[1] ** 2
                                                   * LL[2] ** 2)
cc = 2 * pi / LL


def computeVort(x, y, z):
    wx = coeff * sin(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2])
    wy = coeff * cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])
    wz = coeff * cos(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2])
    return wx, wy, wz


# ref. field
def computeRef(x, y, z, t):
    refx = -2. * pi / LL[1] * \
        (cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[2] * (cos(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2]))

    refy = -2. * pi / LL[2] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        + 2. * pi / LL[0] * (sin(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2]))

    refz = -2. * pi / LL[0] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[1] * (sin(x * cc[0]) * cos(y * cc[1]) * cos(z * cc[2]))

    return refx, refy, refz


def computeVort2D(x, y):
    # todo ...
    wx = coeff * sin(x * cc[0]) * sin(y * cc[1])
    wy = coeff * cos(x * cc[0]) * sin(y * cc[1])
    return wx, wy


# ref. field
def computeRef2D(x, y, t):
    # todo : find a proper ref
    refx = 1.
    refy = 1.
    return refx, refy


def test_Poisson3D():
    dom = pp.Box(dimension=dim, length=LL)

    # Fields
    velocity = pp.Field(domain=dom, isVector=True, name='Velocity')
    vorticity = pp.Field(domain=dom, formula=computeVort,
                         name='Vorticity', isVector=True)

    resol = [33, 33, 33]
    # Definition of the Poisson operator
    poisson = Poisson(velocity, vorticity,
                      resolutions={velocity: resol, vorticity: resol})

    # Analytic operator to compute the reference field
    ref = pp.Field(domain=dom, name='reference', isVector=True)
    refOp = Analytic(ref, formula=computeRef,
                     resolutions={ref: resol})

    simu = Simulation()
    refOp.setUp()
    poisson.setUp()
    refOp.apply(simu)
    vorticity.initialize()
    poisson.apply(simu)
    assert np.allclose(ref.norm(), velocity.norm())
    poisson.finalize()
    refD = ref.discreteFields.values()[0]
    vd = velocity.discreteFields.values()[0]

    assert np.allclose(ref.norm(), velocity.norm())
    for i in range(dom.dimension):
        assert np.allclose(vd[i], refD[i])


## def test_Poisson2D():
##     dom = pp.Box(dimension=2, length=[2. * pi, 2. * pi], origin=[0., 0.])

##     # Fields
##     velocity = pp.Field(domain=dom, isVector=True, name='Velocity')
##     vorticity = pp.Field(domain=dom, formula=computeVort2D,
##                          name='Vorticity', isVector=True)

##     resol = [33, 33]
##     # Definition of the Poisson operator
##     poisson = Poisson(velocity, vorticity,
##                       resolutions={velocity: resol, vorticity: resol})

##     # Analytic operator to compute the reference field
##     ref = pp.Field(domain=dom, name='reference', isVector=True)
##     refOp = Analytic(ref, formula=computeRef2D,
##                      resolutions={ref: resol})

##     simu = Simulation()
##     refOp.setUp()
##     poisson.setUp()
##     refOp.apply(simu)
##     vorticity.initialize()
##     poisson.apply(simu)
## #    assert np.allclose(ref.norm(), velocity.norm())
##     poisson.finalize()
##  #   refD = ref.discreteFields.values()[0]
##  #   vd = velocity.discreteFields.values()[0]

## #    assert np.allclose(ref.norm(), velocity.norm())
## #    for i in range(dom.dimension):
## #        assert np.allclose(vd[i], refD[i])
