!> @addtogroup part
!! @{
!------------------------------------------------------------------------------
!
! MODULE: advec_common
!
!
! DESCRIPTION:
!> The module ``advec_common'' gather function and subroutines used to advec scalar
!! which are not specific to a direction
!! @details
!! This module gathers functions and routines used to advec scalar which are not
!! specific to a direction. This is a parallel implementation using MPI and
!! the cartesien topology it provides. It also contains the variables common to
!! the solver along each direction and other generic variables used for the
!! advection based on the particle method.
!!
!! Except for testing purpose, this module is not supposed to be used by the
!! main code but only by the other advection module. More precisly, an final user
!! must only used the generic "advec" module wich contain all the interface to
!! solve the advection equation with the particle method, and to choose the
!! remeshing formula, the dimensionnal splitting and everything else.
!!
!! The module "test_advec" can be used in order to validate the procedures
!! embedded in this module.
!
!> @author
!! Jean-Baptiste Lagaert, LEGI
!
!------------------------------------------------------------------------------

module advec_common

    ! Velocity interpolation at particle position
    use advec_common_velo  ,only:AC_velocity_interpol_group, AC_velocity_interpol_no_com
    ! Particles remeshing
    use advec_common_remesh,only: AC_setup_init,                &
            & AC_setup_alongX, AC_setup_alongY, AC_setup_alongZ,&
            & AC_remesh_lambda_group, AC_remesh_limit_lambda_group, AC_remesh_Mprime_group

    implicit none

end module advec_common
