# -*- coding: utf-8 -*-
"""
@package parmepy.operator.Stretching

Penalization operator representation
"""
from parmepy.operator.continuous import Operator
from parmepy.operator.differentialOperator_d import DifferentialOperator_d


class DifferentialOperator(Operator):
    """
    Differential operator
    """

    def __init__(self, field1, field2, choice=None,
                 resolutions=None, method='',
                 **other_config):
        """
        Constructor.
        Create a Differential operator from given velocity and vorticity variables.

        @param field1 : velocity variable.
        @param field2 : vorticity variable.
        @param choice : choice of the differential operation in : curl , divWU , gradV, gradS
        """
        Operator.__init__(self)
        self.field1 = field1
        self.field2 = field2
        self.choice = choice
        self.addVariable([field1, field2])
        self.resolutions = resolutions
        self.method = method
        self.config = other_config

    def setUp(self):
        """
        Stretching operator discretization method.
        Create a discrete Stretching operator from given specifications.

        @param field1 : Index of velocity discretisation to use.
        @param field2 : Index of vorticity discretisation to use.
        @param topology : topology.
        """
        Operator.setUp(self)
        self.discreteOperator = DifferentialOperator_d(self, method=self.method,
                                                       **self.config)
        self.discreteOperator.setUp()

    def __str__(self):
        """ToString method"""
        s = " Differencial operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : DifferentialOperator"
    print DifferentialOperator.__doc__
